/*
Copyright 1988 by the Massachusetts Institute of Technology
                  and Apollo Computer, Inc. and
                  heirs, successors, and assignees.

Permission to use, copy, modify, and distribute this
software and its documentation for any purpose and without
fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright
notice and this permission notice appear in supporting
documentation, and that the names of Apollo or M.I.T. not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.
Apollo and M.I.T. make no representations about the suitability of
this software for any purpose.  It is provided "as is"
without express or implied warranty.
*/

#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/IntrinsicP.h>
#include "LogoP.h"

static Dimension defDim = 100;

static XtResource resources[] = {
    {XtNwidth, XtCWidth, XtRDimension, sizeof(Dimension),
	XtOffset(Widget,core.width), XtRDimension, (caddr_t)&defDim},
    {XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
	XtOffset(Widget,core.height), XtRDimension, (caddr_t)&defDim},
    {XtNbackground, XtCBackground, XtRPixel, sizeof(Pixel),
	XtOffset(Widget,core.background_pixel), XtRString, "White"},
    {XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
        XtOffset(LogoWidget,logo.fgpixel), XtRString, "Black"},
    {XtNreverseVideo, XtCReverseVideo, XtRBoolean, sizeof (Boolean),
	XtOffset(LogoWidget,logo.reverse_video), XtRString, "FALSE"},
};

static void Initialize(), Realize(), Destroy(), Redisplay(), Resize();
static Boolean SetValues();

LogoClassRec logoClassRec = {
    { /* core fields */
    /* superclass		*/	&widgetClassRec,
    /* class_name		*/	"Logo",
    /* widget_size		*/	sizeof(LogoRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	Realize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	resources,
    /* resource_count		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	XtExposeNoCompress,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	TRUE,
    /* destroy			*/	Destroy,
    /* resize			*/	Resize,
    /* expose			*/	Redisplay,
    /* set_values		*/	SetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
    }
};

WidgetClass logoWidgetClass = (WidgetClass) &logoClassRec;
      
static void makeLogoBitmap ( w )
    LogoWidget w;
{
   Pixmap bitmap;
   XGCValues gcv;
   GC gcfg, gcbg;

   if (w->logo.bitmap) {
     XFreePixmap ( XtDisplay(w), w->logo.bitmap);
     w->logo.bitmap = 0;
   }

   bitmap = XCreatePixmap(
             XtDisplay(w),
             XtWindow(w),
             (unsigned)w->core.width,
             (unsigned)w->core.height,
             1 );

    gcv.foreground = 1;
    gcfg = XCreateGC(XtDisplay(w), bitmap, GCForeground, &gcv);
    gcv.foreground = 0;
    gcbg = XCreateGC(XtDisplay(w), bitmap, GCForeground, &gcv);
    XDrawLogo( XtDisplay(w),
               bitmap, gcfg, gcbg,
               0, 0,
               (unsigned)w->core.width,
               (unsigned)w->core.height
               );
    XFreeGC(XtDisplay(w), gcfg);
    XFreeGC(XtDisplay(w), gcbg);

    w->logo.bitmap = bitmap;

}    

/* ARGSUSED */
static void Initialize (request, new)
    Widget request, new;
{
    LogoWidget w = (LogoWidget)new;
    XGCValues  gcv;

    if (w->logo.reverse_video) {
	Pixel fg = w->logo.fgpixel;
	Pixel bg = w->core.background_pixel;

	if (w->core.border_pixel == fg)
	    w->core.border_pixel = bg;
	w->logo.fgpixel = bg;
	w->core.background_pixel = fg;
    }

    gcv.foreground = w->logo.fgpixel;
    gcv.background = w->core.background_pixel;
    gcv.graphics_exposures = False;
    w->logo.foreGC = XtGetGC(
           (Widget)w, 
           GCForeground | 
           GCBackground | 
           GCGraphicsExposures,
           &gcv);

    w->logo.bitmap = 0;
}

static void Realize (gw, valueMask, attrs)
    Widget gw;
    XtValueMask *valueMask;
    XSetWindowAttributes *attrs;
{
    XtCreateWindow( gw, InputOutput, (Visual *)CopyFromParent,
		    *valueMask, attrs);
}

static void Destroy (gw)
    Widget gw;
{
    LogoWidget w = (LogoWidget) gw;

    XtDestroyGC (w->logo.foreGC);
    if (w->logo.bitmap ) {
      XFreePixmap ( 
           XtDisplay((Widget)w), 
           w->logo.bitmap);
      w->logo.bitmap = 0;
    }
}

/* ARGSUSED */
static void Redisplay (gw, event, region)
    Widget gw;
    XEvent *event;		/* unused */
    Region region;		/* unused */
{
    LogoWidget w = (LogoWidget) gw;

    if (w->logo.bitmap == None) makeLogoBitmap ( w );

    if (w->core.visible)  XCopyPlane (
          XtDisplay(w),
          w->logo.bitmap,
          XtWindow(w), 
          w->logo.foreGC,
          event->xexpose.x,     event->xexpose.y,
          event->xexpose.width, event->xexpose.height,
          event->xexpose.x,     event->xexpose.y,
          1 );
}

static void Resize (gw)
    Widget gw;
{
    LogoWidget w = (LogoWidget) gw;
    if (w->logo.bitmap) {
     XFreePixmap ( XtDisplay(w), w->logo.bitmap);
     w->logo.bitmap = None;
   }
}

static Boolean SetValues (gcurrent, grequest, gnew)
    Widget gcurrent, grequest, gnew;
{
    LogoWidget current = (LogoWidget) gcurrent;
    LogoWidget new = (LogoWidget) gnew;
    Boolean redisplay = FALSE;
    XGCValues	gcv;

    if ((new->logo.fgpixel != current->logo.fgpixel) ||
	(new->core.background_pixel != current->core.background_pixel)) {

	XtReleaseGC (current, current->logo.foreGC);
        gcv.foreground = new->logo.fgpixel;
        gcv.background = new->core.background_pixel;
        gcv.graphics_exposures = False;
        new->logo.foreGC = XtGetGC(
           new, 
           GCForeground | 
           GCBackground | 
           GCGraphicsExposures,
           &gcv);

	redisplay = TRUE;
    }
   
   return (redisplay);
}
